
#ifdef GL_ES
precision mediump float;
#endif

varying vec2 v_texCoord;
varying vec4 v_fragmentColor;

//是否开启高亮
uniform int u_isOpen;
//色相
uniform float u_hue;
//饱和度
uniform float u_saturation;
//明度
uniform float u_value;


//RGB to HSV
vec3 RGBConvertToHSV(vec3 rgb) {
    float R = rgb.x;
    float G = rgb.y;
    float B = rgb.z;
    vec3 hsv;
    float max1 = max(R, max(G, B));
    float min1 = min(R, min(G, B));
    
    if (R == max1)
    {
        hsv.x = (G - B) / (max1 - min1);
    }
    if (G == max1)
    {
        hsv.x = 2.0 + (B - R) / (max1 - min1);
    }
    if (B == max1)
    {
        hsv.x = 4.0 + (R - G) / (max1 - min1);
    }
    
    hsv.x = hsv.x * 60.0;
    
    if (hsv.x < 0.0) {
        hsv.x = hsv.x + 360.0;
    }
    
    hsv.z = max1;
    hsv.y = (max1 - min1) / max1;
    return hsv;
}

//HSV to RGB
vec3 HSVConvertToRGB(vec3 hsv) {
    float R, G, B;
    if (hsv.y == 0.0) {
        R = G = B = hsv.z;
    } else {
        hsv.x = hsv.x / 60.0;
        int i = int(hsv.x);
        float f = hsv.x - float(i);
        float a = hsv.z * (1.0 - hsv.y );
        float b = hsv.z * (1.0 - (hsv.y * f));
        float c = hsv.z * (1.0 - (hsv.y * (1.0 - f)));
        if (i == 0) {
            R = hsv.z;
            G = c;
            B = a;
        } else if (i == 1) {
            R = b;
            G = hsv.z;
            B = a;
        } else if (i == 2) {
            R = a;
            G = hsv.z;
            B = c;
        } else if (i == 3) {
            R = a;
            G = b;
            B = hsv.z;
        } else if (i == 4) {
            R = c;
            G = a;
            B = hsv.z;
        } else {
            R = hsv.z;
            G = a;
            B = b;
        }
    }
    return vec3(R, G, B);
}


void main() {
    
    vec4 sampleColor = texture2D(CC_Texture0, v_texCoord) * v_fragmentColor;
    if (u_isOpen != 0) {
        vec3 colorHSV;
        //转换为HSV
        colorHSV.xyz = RGBConvertToHSV(sampleColor.xyz);
        //调整偏移Hue值
        colorHSV.x += u_hue;
        //超过360的值从0开始
        colorHSV.x = (float(mod (colorHSV.x, 360.0)));
        //调整饱和度
        colorHSV.y *= u_saturation;
        //明度
        colorHSV.z *= u_value;
        //将调整后的HSV，转换为RGB颜色
        sampleColor.xyz = HSVConvertToRGB(colorHSV.xyz);
    }
    
    gl_FragColor = sampleColor;
}
